@@ -17,7 +17,7 @@
  */
 #include <stdio.h>
 #include <ctype.h>
-#ifdef SEQUENT
+#ifdef sequent
 #include <strings.h>
 #define strrchr rindex
 #else
@@ -30,21 +30,14 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <pwd.h>
-#include "identd.h"
 
-/*
- * To resolve external references that are usually resolved
- * from identd.c
- */
-char *path_unix = NULL;
-char *path_kmem = NULL;
-int lport = 0;
-int fport = 0;
+#include "pidentd.h"
 
-int debug_flag =1;
-int syslog_flag = 0;
 
-#ifdef SEQUENT
+int debug = 0;
+
+
+#ifdef sequent
 char *strtok(str, dels)
     char *str;
     char *dels;
@@ -74,20 +67,44 @@ char *strtok(str, dels)
 #endif
 
 int
-main()
+main(int argc, char **argv)
 {
+    int c;
     char buf[500];
-    if (k_open() < 0) {
-	fprintf(stderr,"k_open failed\n");
-	exit(1);
+    int try;
+    void *kp;
+
+
+    while ((c = getopt(argc, argv, "d")) != -1) {
+        switch (c)
+	{
+	  case 'd':
+	    debug++;
+	    break;
+	  default:
+	    fprintf(stderr, "Usage: %s [-d]\n", argv[0]);
+	    return 1;
+	}
     }
+
+    if (ka_init()) {
+	fprintf(stderr,
+		"%s: OS version mismatch - compiled for: %s\n",
+		argv[0], osinfo_build);
+	return 1;
+    }
+    if (ka_open(&kp) < 0) {
+        fprintf(stderr, "%s: Cannot open kernel memory\n", argv[0]);
+	return 1;
+    }
+
     while (fgets(buf,sizeof(buf),stdin)) {
 	char *loc, *rem, *tmp;
 	unsigned short lport, fport;
 	struct in_addr faddr, laddr;
-	int uid;
-	char buf2[sizeof(buf)];
+        struct kernel k;
 	struct passwd *pwd;
+	char buf2[sizeof(buf)];
 
 	strcpy(buf2,buf);
 
@@ -121,18 +138,45 @@ main()
 	fport = atoi(tmp);
 	faddr.s_addr = inet_addr(rem);
 
-	buf2[tmp - buf + strlen(tmp)] = 0;
-	if (k_getuid(&faddr, htons(fport), &laddr, htons(lport), &uid)) {
-	    fprintf(stderr,"*unknown*\t%s\t%d\t\t%s\t%d\n", loc, lport, rem, fport);
-	    continue;
-	}
+	k.status = -1;
+	k.euid = -1;
+	k.ruid = -1;
+	k.pid = -1;
+	k.cmd = NULL;
+	k.argv = NULL;
+        k.remote.sin_port=htons(fport);
+	k.local.sin_port=htons(lport);
+	k.remote.sin_addr=faddr;
+	k.local.sin_addr=laddr;
+	for (try = 0; try < 5; try++)
+	  {
+	    k.status = ka_lookup(kp, &k);
+	    if (k.status>0)
+	      break;
+	  }
 
-	pwd = getpwuid(uid);
+	if (k.status<=0)
+	  {
+	    fprintf(stderr,"%-9.9s\t%-13s\t%-4d\t%-13s\t%-4d\n", 
+		    "*unknown*", loc, lport, rem, fport);
+	    continue;
+	  }
+
+	pwd = getpwuid(k.ruid);
 	if (pwd)
-	    printf("%-8.8s", pwd->pw_name);
+	    printf("%-8.8s[%d]", pwd->pw_name, k.euid);
 	else
-	    printf("%-8.8d", uid);
-	printf(" \t%s\t%d\t\t%s\t%d\n", loc, lport, rem, fport);
+	    printf("%-8.8d[%d]", k.ruid, k.euid);
+	if (k.cmd)
+	{
+	  printf (" \t%-13s\t%-4d\t%-13s\t%-4d\tPID=%d\tCMD=%s\tCMD+ARG=%s\n",
+		  loc, lport, rem, fport, k.pid, k.cmd, k.argv);
+	  s_free(k.cmd); s_free(k.argv);
+	}
+	else
+	  printf (" \t%-13s\t%-4d\t%-13s\t%-4d pid[%d]\n",
+		  loc, lport, rem, fport, k.pid);
+
     }
 
     return 0;
